@@ -1,5 +1,5 @@
 // Copyright (C) 2021 UOS Technology Co., Ltd.
-// SPDX-FileCopyrightText: 2022 - 2023 UnionTech Software Technology Co., Ltd.
+// SPDX-FileCopyrightText: 2022 - 2024 UnionTech Software Technology Co., Ltd.
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -13,6 +13,7 @@ extern "C" {
 #include "fs_buf.h"
 #include "walkdir.h"
 #include "resourceutil .h"
+#include "utils.h"
 }
 
 #include <ddiskmanager.h>
@@ -24,10 +25,21 @@ extern "C" {
 #include <QStandardPaths>
 #include <QRegularExpression>
 #include <QTimer>
+#include <QDBusMessage>
+
+#include <PolkitQt1/Authority>
+
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
+#include <QTextCodec>
+#else
+#include <QStringConverter>
+#endif
 
 #include <unistd.h>
 #include <sys/time.h>
 #include <sys/stat.h>
+#include <sys/vfs.h>
+#include <linux/magic.h>
 
 Q_LOGGING_CATEGORY(logN, "anything.normal.manager", DEFAULT_MSG_TYPE)
 Q_LOGGING_CATEGORY(logC, "anything.changes.manager", DEFAULT_MSG_TYPE)
@@ -57,6 +69,35 @@ static QString _getCacheDir()
     return cachePath;
 }
 
+// 尝试将用户输入的绑定挂载下的路径, 转化为原始挂载下的路径
+// 如 /home -> /data/home
+// https://pms.uniontech.com/bug-view-303925.html
+QString convertPathIntoMountPoint(const QStringList &mountPoints, const QString &path)
+{
+    if (path == "/")
+        return path;
+
+    for (const QString &mountPoint : mountPoints) {
+        if (mountPoint == "/")
+            continue;
+        if (path.startsWith(mountPoint)) {
+            return path;
+        }
+    }
+
+    for (const QString &mountPoint : mountPoints) {
+        char* pathInMountPoint = find_matching_dir_by_cache (
+            mountPoint.toLocal8Bit().constData(), path.toLocal8Bit().constData());
+        if (pathInMountPoint) {
+            QString newPath = QString::fromLocal8Bit(pathInMountPoint);
+            g_free(pathInMountPoint);
+            return newPath;
+        }
+    }
+
+    return path;
+}
+
 class _LFTManager : public LFTManager {};
 Q_GLOBAL_STATIC(_LFTManager, _global_lftmanager)
 typedef QMap<QString, fs_buf*> FSBufMap;
@@ -74,7 +115,12 @@ static QSet<fs_buf*> fsBufList()
     if (!_global_fsBufMap.exists())
         return QSet<fs_buf*>();
 
-    return _global_fsBufMap->values().toSet();
+    auto values = _global_fsBufMap->values();
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
+    return values.toSet();
+#else
+    return QSet<fs_buf*>(values.begin(), values.end());
+#endif
 }
 
 static void clearFsBufMap()
@@ -130,6 +176,10 @@ static void cleanDirtyLFTFiles()
 
 LFTManager::~LFTManager()
 {
+    cpu_monitor_quit.unlock();
+    cpu_monitor_thread->wait();
+    delete cpu_monitor_thread;
+
     sync();
     clearFsBufMap();
     // 删除剩余脏文件(可能是sync失败)
@@ -150,6 +200,10 @@ QString LFTManager::cacheDir()
 
 QByteArray LFTManager::setCodecNameForLocale(const QByteArray &codecName)
 {
+    if (!checkAuthorization())
+        return QByteArray();
+
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
     const QTextCodec *old_codec = QTextCodec::codecForLocale();
 
     if (codecName.isEmpty())
@@ -160,6 +214,17 @@ QByteArray LFTManager::setCodecNameForLocale(const QByteArray &codecName)
     nDebug() << codecName << "old:" << old_codec->name();
 
     return old_codec->name();
+#else
+    // 在Qt6中，使用QStringConverter来处理编码
+    // 获取当前的编码
+    QStringConverter::Encoding old_encoding = QStringConverter::encodingForName(QString::fromLatin1(codecName)).value_or(QStringConverter::System);
+
+    QString oldCodecName = QString::fromLatin1(QStringConverter::nameForEncoding(old_encoding));
+    // 由于QStringConverter没有setDefaultEncoding，这里只记录旧编码
+    nDebug() << codecName << "old:" << oldCodecName;
+
+    return oldCodecName.toLatin1();
+#endif
 }
 
 void LFTManager::onFileChanged(QList<QPair<QByteArray, QByteArray>> &actionList)
@@ -251,7 +316,7 @@ static QString getLFTFileByPath(const QString &path, bool autoIndex)
     if (cache_path.isEmpty())
         return QString();
 
-    return cache_path + "/" + QString::fromLocal8Bit(lft_file_name.toPercentEncoding(":", "/"));
+    return cache_path + "/" + QString::fromLocal8Bit(lft_file_name.toPercentEncoding(":", "/").constData());
 }
 
 static bool allowablePath(LFTManager *manager, const QString &path)
@@ -311,6 +376,9 @@ static void removeBuf(fs_buf *buf, bool &removeLFTFile)
 
 bool LFTManager::addPath(QString path, bool autoIndex)
 {
+    if (!checkAuthorization())
+        return false;
+
     nDebug() << path << autoIndex;
 
     if (!path.startsWith("/")) {
@@ -349,7 +417,7 @@ bool LFTManager::addPath(QString path, bool autoIndex)
 
     // 保存信息，用于判断索引是否正在构建
     for (const QByteArray &path_raw : path_list) {
-        const QString &path = QString::fromLocal8Bit(path_raw);
+        const QString &path = QString::fromLocal8Bit(path_raw.constData());
 
         (*_global_fsWatcherMap)[path] = watcher;
     }
@@ -366,7 +434,7 @@ bool LFTManager::addPath(QString path, bool autoIndex)
         }
 
         for (const QByteArray &path_raw : path_list) {
-            const QString &path = QString::fromLocal8Bit(path_raw);
+            const QString &path = QString::fromLocal8Bit(path_raw.constData());
 
             if (buf) {
                 // 清理旧数据
@@ -407,6 +475,9 @@ bool LFTManager::addPath(QString path, bool autoIndex)
 
 bool LFTManager::removePath(const QString &path)
 {
+    if (!checkAuthorization())
+        return false;
+
     nDebug() << path;
 
     if (fs_buf *buf = _global_fsBufMap->take(path)) {
@@ -453,6 +524,36 @@ static QPair<QString, fs_buf*> getFsBufByPath(const QString &path)
         nWarning() << "getFsBufByPath findMountPointByPath NULL for:" << path;
         return QPair<QString, fs_buf*>();
     }
+
+    // 适配磐石系统, 将 /sysroot/dirname 的挂载点, 从 / 调整为 /sysroot
+    if (("/" == mountPoint) && path.startsWith("/sysroot")) {
+        static bool inited = false;
+        static bool convert_mountpoint_from_root_to_sysroot = false;
+
+        if (!inited) {
+            struct statfs root_statfs;
+            struct stat root_stat;
+            struct stat sysroot_stat;
+
+            inited = true;
+            // / 所在文件系统为 ext4 且不是文件系统根目录
+            // 参考 https://docs.kernel.org/filesystems/ext4/overview.html#special-inodes
+            if (statfs("/", &root_statfs) == 0 &&
+                root_statfs.f_type == EXT4_SUPER_MAGIC &&
+                lstat("/", &root_stat) == 0 &&
+                root_stat.st_ino != 2 &&
+                // /sysroot 与 / 同分区且是根目录
+                lstat("/sysroot", &sysroot_stat) == 0 &&
+                sysroot_stat.st_dev == root_stat.st_dev &&
+                sysroot_stat.st_ino == 2) {
+                convert_mountpoint_from_root_to_sysroot = true;
+            }
+        }
+
+        if (convert_mountpoint_from_root_to_sysroot)
+            mountPoint = "/sysroot";
+    }
+
     QPair<QString, fs_buf*> buf_pair;
     fs_buf *buf = _global_fsBufMap->value(mountPoint);
     if (buf) {
@@ -478,7 +579,10 @@ static QPair<QString, fs_buf*> getFsBufByPath(const QString &path)
 
 bool LFTManager::hasLFT(const QString &path) const
 {
-    auto buff_pair = getFsBufByPath(path);
+    QStringList mountPoints = allPath();
+    QString pathIntoMountPoint = convertPathIntoMountPoint(mountPoints, path);
+
+    auto buff_pair = getFsBufByPath(pathIntoMountPoint);
     return !buff_pair.first.isEmpty();
 }
 
@@ -489,6 +593,9 @@ bool LFTManager::lftBuinding(const QString &path) const
 
 bool LFTManager::cancelBuild(const QString &path)
 {
+    if (!checkAuthorization())
+        return false;
+
     nDebug() << path;
 
     if (QFutureWatcher<fs_buf*> *watcher = _global_fsWatcherMap->take(path)) {
@@ -541,6 +648,9 @@ QStringList LFTManager::hasLFTSubdirectories(QString path) const
 // 重新从磁盘加载lft文件
 QStringList LFTManager::refresh(const QByteArray &serialUriFilter)
 {
+    if (!checkAuthorization())
+        return QStringList();
+
     nDebug() << serialUriFilter;
 
     const QString &cache_path = cacheDir();
@@ -590,7 +700,7 @@ QStringList LFTManager::refresh(const QByteArray &serialUriFilter)
         }
 
         for (const QByteArray &path_raw : pathList) {
-            const QString path = QString::fromLocal8Bit(path_raw);
+            const QString path = QString::fromLocal8Bit(path_raw.constData());
 
             path_list << path;
 
@@ -611,6 +721,9 @@ QStringList LFTManager::refresh(const QByteArray &serialUriFilter)
 
 QStringList LFTManager::sync(const QString &mountPoint)
 {
+    if (!checkAuthorization())
+        return QStringList();
+
     nDebug() << mountPoint;
 
     QStringList path_list;
@@ -705,16 +818,19 @@ enum SearchError
 
 QStringList LFTManager::insertFileToLFTBuf(const QByteArray &file)
 {
+    if (!checkAuthorization())
+        return QStringList();
+
     cDebug() << file;
 
-    auto buff_pair = getFsBufByPath(QString::fromLocal8Bit(file));
+    auto buff_pair = getFsBufByPath(QString::fromLocal8Bit(file.constData()));
     QStringList root_path_list;
 
     QString mount_path = buff_pair.first;
     if (mount_path.isEmpty())
         return root_path_list;
 
-    QFileInfo info(QString::fromLocal8Bit(file));
+    QFileInfo info(QString::fromLocal8Bit(file.constData()));
     bool is_dir = info.isDir();
 
     fs_buf *buf = buff_pair.second;
@@ -746,8 +862,11 @@ QStringList LFTManager::insertFileToLFTBuf(const QByteArray &file)
     } else {
         if (r == ERR_NO_MEM) {
             cWarning() << "Failed(No Memory):" << mount_path;
+        } else if (r == ERR_PATH_EXISTS) {
+            /* 由于事件合并的原因, 会经常导致报告此类错误. 由于它不属于程序问题, 特此降低日志等级 */
+            cDebug() << "Failed(Path Exists):" << mount_path;
         } else {
-            cWarning() << "Failed:" << mount_path << ", result:" << r;
+            cDebug() << "Failed:" << mount_path << ", result:" << r;
         }
     }
 
@@ -756,9 +875,12 @@ QStringList LFTManager::insertFileToLFTBuf(const QByteArray &file)
 
 QStringList LFTManager::removeFileFromLFTBuf(const QByteArray &file)
 {
+    if (!checkAuthorization())
+        return QStringList();
+
     cDebug() << file;
 
-    auto buff_pair = getFsBufByPath(QString::fromLocal8Bit(file));
+    auto buff_pair = getFsBufByPath(QString::fromLocal8Bit(file.constData()));
     QStringList root_path_list;
 
     QString mount_path = buff_pair.first;
@@ -805,9 +927,12 @@ QStringList LFTManager::removeFileFromLFTBuf(const QByteArray &file)
 
 QStringList LFTManager::renameFileOfLFTBuf(const QByteArray &oldFile, const QByteArray &newFile)
 {
+    if (!checkAuthorization())
+        return QStringList();
+
     cDebug() << oldFile << newFile;
 
-    auto buff_pair = getFsBufByPath(QString::fromLocal8Bit(newFile));
+    auto buff_pair = getFsBufByPath(QString::fromLocal8Bit(newFile.constData()));
     QStringList root_path_list;
 
     QString mount_path = buff_pair.first;
@@ -862,6 +987,9 @@ QStringList LFTManager::renameFileOfLFTBuf(const QByteArray &oldFile, const QByt
 
 void LFTManager::quit()
 {
+    if (!checkAuthorization())
+        return;
+
     qApp->quit();
 }
 
@@ -905,6 +1033,9 @@ QStringList LFTManager::parallelsearch(const QString &path, quint32 startOffset,
 
 void LFTManager::setAutoIndexExternal(bool autoIndexExternal)
 {
+    if (!checkAuthorization())
+        return;
+
     if (this->autoIndexExternal() == autoIndexExternal)
         return;
 
@@ -922,6 +1053,9 @@ void LFTManager::setAutoIndexExternal(bool autoIndexExternal)
 
 void LFTManager::setAutoIndexInternal(bool autoIndexInternal)
 {
+    if (!checkAuthorization())
+        return;
+
     if (this->autoIndexInternal() == autoIndexInternal)
         return;
 
@@ -939,6 +1073,9 @@ void LFTManager::setAutoIndexInternal(bool autoIndexInternal)
 
 void LFTManager::setLogLevel(int logLevel)
 {
+    if (!checkAuthorization())
+        return;
+
     nDebug() << "setLogLevel:" << logLevel;
 
     QString rules;
@@ -986,11 +1123,16 @@ static QStringList removeLFTFiles(const QByteArray &serialUriFilter = QByteArray
 LFTManager::LFTManager(QObject *parent)
     : QObject(parent)
 {
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
     // ascii编码支持内容太少, 此处改为兼容它的utf8编码
     if (QTextCodec::codecForLocale() == QTextCodec::codecForName("ASCII")) {
         QTextCodec::setCodecForLocale(QTextCodec::codecForName("UTF-8"));
         nDebug() << "reset the locale codec to UTF-8";
     }
+#else
+    // 6.0版本以上, 默认使用UTF-8编码，无需额外处理
+    nDebug() << "Qt6 uses UTF-8 encoding by default";
+#endif
 
     /*解决在最开始的10分钟内搜索不到问题 92168*/
     // 延迟 1s 尝试加载索引
@@ -1025,13 +1167,17 @@ LFTManager::LFTManager(QObject *parent)
     sync_timer->setInterval(10 * 60 * 1000);
     sync_timer->start();
 
-    // 使用CPU资源和控制; 10秒检测一次，连续3次>85%, 则使用cgroup限制50%，连续3次<30%，解除限制。
+    // 使用CPU资源和控制; 10秒检测一次。
     cpu_row_count = 0;
     cpu_limited = false;
-    QTimer *resource_timer = new QTimer(this);
-    connect(resource_timer, &QTimer::timeout, this, &LFTManager::_cpuLimitCheck);
-    resource_timer->setInterval(10 * 1000);
-    resource_timer->start();
+    cpu_monitor_quit.lock();
+    cpu_monitor_thread = QThread::create(std::function<void ()> ([this]() {
+        while (!this->cpu_monitor_quit.tryLock(10 * 1000)) {
+            this->_cpuLimitCheck();
+        }
+        this->cpu_monitor_quit.unlock();
+    }));
+    cpu_monitor_thread->start();
 
     // 创建索引结束解除CPU限定
     connect(this, &LFTManager::buildFinished, this, [this]() {
@@ -1094,9 +1240,9 @@ void LFTManager::_cpuLimitCheck()
             nWarning() << "Limited, long time high CPU usage: " << current_cpu;
 
             struct stat statbuf;
-            if (stat("/tmp/anything_disable_abort", &statbuf)) {
-                nWarning() << "Abort for high CPU usage.";
-                abort();
+            if (stat("/tmp/anything_disable_exit", &statbuf)) {
+                nWarning() << "Exit for high CPU usage.";
+                _exit(1);
             }
         } else if (current_cpu < low_use) {
             QProcess::startDetached(cmd);
@@ -1167,23 +1313,27 @@ void LFTManager::_cleanAllIndex()
     }
 }
 
-static QString getRootMountPoint(const DBlockDevice *block)
+QString getRootMountPoint(const DBlockDevice *block)
 {
-    const QByteArrayList &mount_points = block->mountPoints();
-    if(mount_points.size() == 1) {
-        return QString::fromLocal8Bit(mount_points.first());
+    /* Perform conversions early to detect potential crashes as early as possible */
+    QByteArrayList mountPointsByteArray = block->mountPoints();
+    QStringList mountPoints;
+
+    foreach (const QByteArray &item, mountPointsByteArray) {
+        mountPoints.append(QString::fromLocal8Bit(item.constData()));
+    }
+    if(mountPoints.size() == 1) {
+        return mountPoints.first();
     }
 
-    const auto mount_point_infos = deepin_anything_server::MountCacher::instance()->getRootsByPoints(mount_points);
-    for (QByteArray mount_point : mount_points) {
-        const QString root_point = mount_point_infos.value(mount_point);
-        if (root_point == "/") {
-            mount_point.chop(1);
-            return QString::fromLocal8Bit(mount_point);
+    QMap<QString, QString> mountPoint2Root = deepin_anything_server::MountCacher::instance()->getRootsByStrPoints(mountPoints);
+    for (const QString &mountPoint : mountPoints) {
+        if (mountPoint2Root.value(mountPoint) == "/") {
+            return mountPoint;
         }
     }
 
-    return QString::fromLocal8Bit(mount_points.first());
+    return mountPoints.first();
 }
 
 void LFTManager::_addPathByPartition(const DBlockDevice *block)
@@ -1214,13 +1364,15 @@ void LFTManager::onMountAdded(const QString &blockDevicePath, const QByteArray &
 {
     nInfo() << blockDevicePath << mountPoint;
 
-    const QString &mount_root = QString::fromLocal8Bit(mountPoint);
+    deepin_anything_server::MountCacher::instance()->updateMountPoints();
+
+    const QString &mount_root = QString::fromLocal8Bit(mountPoint.constData());
     const QByteArray &serial_uri = LFTDiskTool::pathToSerialUri(mount_root);
 
     // 尝试加载此挂载点下的lft文件
     const QStringList &list = refresh(serial_uri.toPercentEncoding(":", "/"));
 
-    if (list.contains(QString::fromLocal8Bit(mountPoint))) {
+    if (list.contains(QString::fromLocal8Bit(mountPoint.constData()))) {
         return;
     }
 
@@ -1241,7 +1393,9 @@ void LFTManager::onMountRemoved(const QString &blockDevicePath, const QByteArray
 {
     nInfo() << blockDevicePath << mountPoint;
 
-    const QString &mount_root = QString::fromLocal8Bit(mountPoint);
+    deepin_anything_server::MountCacher::instance()->updateMountPoints();
+
+    const QString &mount_root = QString::fromLocal8Bit(mountPoint.constData());
 //    const QByteArray &serial_uri = LFTDiskTool::pathToSerialUri(mount_root);
 
     for (const QString &path : hasLFTSubdirectories(mount_root)) {
@@ -1301,7 +1455,7 @@ int LFTManager::_prepareBuf(quint32 *startOffset, quint32 *endOffset, const QStr
     if (!fs_buf)
         return BUILDING_INDEX;
 
-    // new_path 为path在fs_buf中对应的路径
+    // new_path 为搜索路径在fs_buf中对应的路径
     *newpath = QString(buff_pair.first);
 
     if (*startOffset == 0 || *endOffset == 0) {
@@ -1454,6 +1608,8 @@ QStringList LFTManager::_enterSearch(const QString &opath, const QString &keywor
         // make sure this search path not end with '/' if it's not the root /
         path.chop(1);
     }
+    QStringList mountPoints = allPath();
+    path = convertPathIntoMountPoint(mountPoints, path);
     nInfo() << maxCount << startOffset << endOffset << path << keyword << rules;
 
     void *buf = nullptr;
@@ -1620,3 +1776,23 @@ int LFTManager::_doSearch(void *vbuf, quint32 maxCount, const QString &path, con
 
     return total;
 }
+
+bool LFTManager::checkAuthorization(void)
+{
+    if (!calledFromDBus())
+        return true;
+
+    QString actionId("com.deepin.anything");
+    QString appBusName = message().service();
+    PolkitQt1::Authority::Result result;
+
+    result = PolkitQt1::Authority::instance()->checkAuthorizationSync(actionId,
+                                                                      PolkitQt1::SystemBusNameSubject(appBusName),
+                                                                      PolkitQt1::Authority::AllowUserInteraction);
+    if (result == PolkitQt1::Authority::Yes) {
+        return true;
+    }else {
+        sendErrorReply(QDBusError::AccessDenied);
+        return false;
+    }
+}
